//+------------------------------------------------------------------+
//|                                                 IndicatorFactory |
//|                                   Copyright 2021, Niklas Schlimm |
//|                                      https://github.com/nschlimm |
//+------------------------------------------------------------------+
#include <Generic/HashMap.mqh>
#include <Indicators/Indicators.mqh>

//+------------------------------------------------------------------+
//| Factory to share indicators of same configs among clients        |
//+------------------------------------------------------------------+
class IndicatorFactory
  {
private:
   static CHashMap<string, CIndicator*>* m_indicators;
   static CIndicators* m_indicatorsCreator;

public:
   IndicatorFactory();
   ~IndicatorFactory();
   static CIndicator* getInstance(string p_symbol, ENUM_INDICATOR p_indicator, ENUM_TIMEFRAMES p_tf, MqlParam &p_params[]);
   static void destroy();
  };
//+------------------------------------------------------------------+
//| Constructor                                                      |
//+------------------------------------------------------------------+
IndicatorFactory::IndicatorFactory()
  {
  }
//+------------------------------------------------------------------+
//| Destructor                                                       |
//+------------------------------------------------------------------+
IndicatorFactory::~IndicatorFactory()
  {
     destroy();
  }
//+------------------------------------------------------------------+
//| Clean up memory                                                  |
//+------------------------------------------------------------------+
void IndicatorFactory::destroy()
{
     CIndicator* indicators[];
     string names[];
     m_indicators.CopyTo(names,indicators);
     for (int i=0;ArraySize(indicators);i++)
     {
        delete(indicators[i]);
     }
     delete(m_indicators);
     delete(m_indicatorsCreator);
}
//+------------------------------------------------------------------+
//| Static initializers                                              |
//+------------------------------------------------------------------+
CHashMap<string, CIndicator*>* IndicatorFactory::m_indicators = new CHashMap<string, CIndicator*>();
CIndicators* IndicatorFactory::m_indicatorsCreator = new CIndicators();
//+------------------------------------------------------------------+
//| Create and re-use indicator instances. Main method for clients   |
//| that work with multiple indicators.                              |
//+------------------------------------------------------------------+
CIndicator* IndicatorFactory::getInstance(string p_symbol, ENUM_INDICATOR p_indicator, ENUM_TIMEFRAMES p_tf, MqlParam &p_params[])
  {
   string indicatorKey=p_symbol+"_"+EnumToString((ENUM_INDICATOR)p_indicator)+"_"+EnumToString((ENUM_TIMEFRAMES)p_tf)+"_";
   for(int i = 0; i<ArraySize(p_params); i++)
     {
      if(p_params[i].type==TYPE_INT)
         indicatorKey=indicatorKey+p_params[i].integer_value;
      if(p_params[i].type==TYPE_DOUBLE)
         indicatorKey=indicatorKey+p_params[i].double_value;
      if(p_params[i].type==TYPE_STRING)
         indicatorKey=indicatorKey+p_params[i].string_value;
     }
   if(IndicatorFactory::m_indicators.ContainsKey(indicatorKey))
     {
      CIndicator* indicator;
      IndicatorFactory::m_indicators.TryGetValue(indicatorKey,indicator);
      return indicator;
     }
   CIndicator* newIndicator = m_indicatorsCreator.Create(p_symbol,p_tf,p_indicator,ArraySize(p_params),p_params);
   IndicatorFactory::m_indicators.Add(indicatorKey,newIndicator);
   return newIndicator;
  }

//+------------------------------------------------------------------+
